/*
    This file is part of the Cheap Matrix library
    Copyright (C) 2000  Nicolas Brodu
    nicolas.brodu@free.fr

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program, see the file COPYING; if not, write
    to the Free Software Foundation, Inc., 675 Mass Ave, Cambridge,
    MA 02139, USA.
*/

// This example illustrates the use of mapper facilities

#include "CheapMatrix.h"

using namespace CheapMatrix;

// Let's create a Hilbert Matrix generator
double hilbert_generator(int i, int j) {return 1.0 / (i+j-1.0);}

// And a cube root function
double cube_root(double x) {return exp(log(x)/3.0);}

main()
{
	// Let's use the Hilbert generator
	Matrix H = generate(hilbert_generator, 4, 4);
	cout << H << endl;

	// And now, map our cube root function to it
	cout << map(cube_root, H) << endl;

	// There are much more possibilities with map. For example you can map
	// a function with multiple arguments on multiple matrices.
	// Please read the documentation to know more about map...
	// Tip: you can also fill with a generator: H.fill(hilbert_generator);
}

/* This program should print
1 0.5 0.333333 0.25
0.5 0.333333 0.25 0.2
0.333333 0.25 0.2 0.166667
0.25 0.2 0.166667 0.142857

1 0.793701 0.693361 0.629961
0.793701 0.693361 0.629961 0.584804
0.693361 0.629961 0.584804 0.550321
0.629961 0.584804 0.550321 0.522758

*/
