/*
    This file is part of the Cheap Matrix library
    Copyright (C) 2000  Nicolas Brodu
    nicolas.brodu@free.fr

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
#include "CM_inverse.h"
#include "lapack_types.h"
#include "proto.h"

extern "C" {
// LU factorization
int PROTO(dgetrf,DGETRF)(integer *m, integer *n, doublereal *a, integer *lda,
	integer *ipiv, integer *info);
// Inverse, using LU
int PROTO(dgetri,DGETRI)(integer *n, doublereal *a, integer *lda, integer
	*ipiv, doublereal *work, integer *lwork, integer *info);
} // extern "C"

// Don't pollute the standard namespace with our definitions
namespace CheapMatrix {

// inverse a matrix using LU factorization
bool inverse(const Matrix &A)
{
	// First do the LU stuff
	integer N = A.cols();
	if (N != A.rows()) return false;  // No need to go further...
	CHECK_MEM_LEAD(inverse,A,lda)
	integer pivot[N];
	integer info = 0;
	PROTO(dgetrf,DGETRF)(&N, &N, &A(1,1), &lda, &pivot[0], &info);
	if (info != 0) {A.release(); return false;}
	// Now inverse the matrix using the decomposition
	ALLOCATE_WORK(inverse, N)
	PROTO(dgetri,DGETRI)(&N, &A(1,1), &lda, &pivot[0], &work[0], &lwork, &info);
	A.release();
	delete work;
	return !info;
}
}
/*// Test
main()
{
	Matrix A(3,3);
	A = 1, 5, 9,
	    4, 7, 8,
	    2, 3, 6;
//	cin >>A;

	bool b = inverse(A);

	cout << "b: " << b << endl;
	cout << "A: " << endl << A;
}
*/